# -*- coding: utf-8 -*-
"""
Created on Tue Apr 5 03:54:34 2023

@author: Napstablook
"""


import pandas as pd
import re
import time
import os
from selenium.webdriver.common.by import By
from selenium import webdriver

browser = webdriver.Chrome()

def select_sh(data): #筛选沪市股票代码
    sh=['6','900']
    lst = [ x for x in data for startcode in sh if x[:3].startswith(startcode)==True ]
    df = pd.DataFrame(lst,columns=['code'])
    return df

def get_sse(code):
    searchbox=browser.find_element(By.ID,'inputCode')
    searchbox.send_keys(code)
    time.sleep(0.7)
    s=browser.find_elements(By.CLASS_NAME,'filter-option-inner-inner')[1]
    s.click()
    annual = browser.find_element(By.LINK_TEXT,'年报')
    annual.click()
    time.sleep(0.3)
    html = browser.find_element(By.CLASS_NAME, 'table-responsive')
    time.sleep(0.3)
    innerHTML = html.get_attribute('innerHTML')
    #open('%s.html' %code,'w',encoding='utf-8').write(innerHTML)
    browser.refresh()
    dt = DisclosureTable_sh(innerHTML)
    data = dt.get_data()
    os.chdir('original')
    data.to_csv(code+'.csv')
    os.chdir('../')
    
class DisclosureTable_sh():
    '''
    解析上交所定期报告页搜索表格
    '''
    def __init__(self, innerHTML):
        self.html = innerHTML
        self.prefix = 'http://www.sse.com.cn'
        p_code=re.compile('<span>(\d{6})</span>')
        p_name=re.compile('<span>[*]?(\w+|-)</span>')
        p_href=re.compile('<a.*?href="(.*?.pdf)".*?>')
        p_title=re.compile('<a.*?><span.*?>(.*?)</span></a>')
        self.get_code  = lambda td: p_code.search(td).group(1)
        self.get_name  = lambda td: p_name.search(td).group(1)
        self.get_href  = lambda td: p_href.search(td).group(1)
        self.get_title = lambda td: p_title.search(td).group(1)
        self.txt_to_df() #调用txt_to_df(self),得到初始化dataframe用于后续匹配

    def txt_to_df(self):
        # html table text to DataFrame
        html = self.html
        p_tr = re.compile('<tr>(.*?)</tr>', re.DOTALL)
        trs = p_tr.findall(html)
        p_td = re.compile('<td.*?>(.*?)</td>', re.DOTALL)
        tds=[p_td.findall(td) for td in trs if p_td.findall(td)!=[]]
        df = pd.DataFrame({'证券代码' : [td[0] for td in tds],
                           '股票简称' : [td[1] for td in tds],
                           '公告标题和链接' : [td[2] for td in tds],
                           '公告时间' : [td[3] for td in tds]})
        self.df_txt=df

    def get_data(self):
        get_code  = self.get_code
        get_name  = self.get_name
        get_href  = self.get_href
        get_title = self.get_title
        prefix = self.prefix
        
        df = self.df_txt
    
        codes   = [get_code(td) for td in df['证券代码']]
        names   = [get_name(td) for td in df['股票简称']]
        links   = [prefix+get_href(td) for td in df['公告标题和链接']]
        pubtime = [td for td in df['公告时间']]
        finy = [int(time[0:4]) for time in pubtime]
        titles = [get_title(td)+str(time)+'发布' for td,time in zip(df['公告标题和链接'],finy)]
        titles0 = [get_title(td) for td in df['公告标题和链接']]
        self.short_names = names
        data = pd.DataFrame({'证券代码':codes,
                             '股票简称':names,
                             '公告标题':titles,
                             '公告链接':links,
                             '公告时间':pubtime,
                             '原标题':titles0})
        
        for index,row in data.iterrows():
            time = row[4]
            title = row[2]
            a = re.search("摘要|取消|英文", title)
            if "年度报告" not in title and "年报" not in title or a != None or int(time[0:4])<int(start[0:4]):
                data = data.drop(index=index)
            elif '修订版' in title:
                data = data.drop(data[data['原标题']==title[0:-5]].index)
        data = data.reset_index(drop = True)
        self.df_data = data
        return data


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 5 03:56:56 2023

@author: Napstablook
"""

import pandas as pd
import fitz
import pdfplumber
import re
import os
import requests


def download_report(code):  #根据找到的链接下载年报
    os.chdir('original')
    df=pd.read_csv(code+'.csv')
    os.chdir('../report')
    os.makedirs(code,exist_ok=True)
    os.chdir(code)
    name = df.iloc[0,2].replace('*','')
    for index,row in df.iterrows():
        title=row[3].replace('*','')
        href=row[4]
        f = requests.get(href)
        with open (name+title+".pdf", "wb") as report:
            report.write(f.content)
    for  index,row in df.iterrows():
        title=row[3].replace('*','')
        with pdfplumber.open("%s%s.pdf"%(name,title)) as pdf:
            page_count = len(pdf.pages)
        if page_count<30: 
           os.remove("%s%s.pdf"%(name,title))
           df1 = df.drop(df[df['公告标题']==title].index)
           df1.to_csv('../../original/%s.csv'%code,index=False)
    os.chdir('../..')

def findata(code,save):  #从年报pdf中提取金融数据
    d = annualreport_data(code)
    fin = d.pdf_to_data()
    if save == 1:        
        name = d.short_names[0].replace('*','')
        os.chdir('./financedata')
        fin.to_csv('%s %s.csv'%(code,name),mode='w')
        os.chdir('../')
    else:
        return fin
  
class annualreport_data():
    def __init__(self,code):
        original_table = pd.read_csv('./original/%s.csv'%(code))
        self.code = code
        self.short_names = original_table['股票简称'].to_list()
        self.titles = original_table['公告标题']
    
    def read_report(self):
        code = self.code
        name = self.short_names[0].replace('*','')
        txt=[]
        for title in self.titles:
            title = title.replace('*','')
            doc = fitz.open('./report/%s/%s%s.pdf'%(code,name,title))
            text_by_page = [doc.get_page_text(i) for i in range(15)]
            text = ' '.join(text_by_page)
            txt.append(text)
        self.txt=txt
        
    def matches(self):
        txt = self.txt
        
        p_year= re.compile('.*?(\d{4}) .*?年度报告.*?')
        p_rev = re.compile('(?<=\n)营业总?收入（?(\w*?)?）?\s?\n?\s*([\d+,.]*)\s\n?')
        p_eps = re.compile('(?<=\n)基本每股收益（元/?／?\n?股）\s?\n?([-\d+,.]*)\s?\n?')
        
        year = [int(p_year.findall(text)[0]) for text in txt]
        dw = [p_rev.search(text).group(1) for text in txt ]
        revenue=[float(p_rev.search(text).group(2).replace(',',''))*10000
                 if w=="万元" or w=="万" 
                 else float(p_rev.search(text).group(2).replace(',',''))
                 for text,w in zip(txt,dw)]
        eps=[p_eps.search(text).group(1) for text in txt]
        return [year,revenue,eps]
    
    def pdf_to_data(self):
        self.read_report()
        data = self.matches()
        df = pd.DataFrame({'Year':data[0],
                           'Revenue':data[1],
                           'Eps':data[2]}).set_index('Year').sort_index().drop_duplicates()
        self.data = df
        return df


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 5 03:27:08 2023

@author: Napstablook
"""

import pandas as pd
import re
import time
import os
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium import webdriver

browser = webdriver.Chrome()

def select_sz(data): #筛选深市股票代码
     sz=['200','300','301','00','080']   #深市股票代码A股是以000开头；深市B股代码是以200开头；中小板股票以002开头；
                                         #深市创业板的股票是以300、301开头(比如第一只创业板股票特锐德)。
                                         #另外新股申购代码以00开头、配股代码以080开头。
     lst = [ x for x in data for startcode in sz if x[:3].startswith(startcode)==True ]
     df = pd.DataFrame(lst,columns=['code'])
     return df
 
def Input_timeNtype(start,end,typ): #时间格式yyyy-mm-dd 种类格式"年度/半年度/一季度报告"
    START = browser.find_element(By.CLASS_NAME,'input-left') 
    END = browser.find_element(By.CLASS_NAME,'input-right')    
    START.send_keys(start)
    END.send_keys(end + Keys.RETURN)
    browser.find_element(By.LINK_TEXT,'请选择公告类别').click()
    browser.find_element(By.LINK_TEXT,typ).click()
    ActionChains(browser).move_by_offset(200, 100).click().perform()

def get_szse(code,count): #通过股票代码查找对应公司年报下载链接
                               #查询公司大于1时count传入1                           
    Searchbox = browser.find_element(By.ID, 'input_code') 

    Searchbox.send_keys(code)
    time.sleep(0.2)
    Searchbox.send_keys(Keys.RETURN)
    time.sleep(0.5)
    html = browser.find_element(By.ID, 'disclosure-table')
    innerHTML = html.get_attribute('innerHTML')
    
    if int(count)==1: #当批量查询股票时，每搜索一个后删去历史选择
        browser.find_elements(By.CLASS_NAME,'icon-remove')[-1].click()
        
    dt = DisclosureTable(innerHTML)
    data = dt.get_data()
    os.chdir('original')
    data.to_csv(code+'.csv',encoding='utf-8-sig')
    os.chdir('../')
 
class DisclosureTable():
    '''
    解析深交所定期报告页搜索表格
    '''
    def __init__(self, innerHTML):
        self.html = innerHTML
        self.prefix = 'https://disc.szse.cn/download'
        self.prefix_href = 'https://www.szse.cn/'
        # 获得证券的代码和公告时间
        p_a = re.compile('<a.*?>(.*?)</a>', re.DOTALL)
        p_span = re.compile('<span.*?>(.*?)</span>', re.DOTALL)
        self.get_code = lambda txt: p_a.search(txt).group(1).strip()
        self.get_time = lambda txt: p_span.search(txt).group(1).strip()
        # 将txt_to_df赋给self
        self.txt_to_df()
        
    def txt_to_df(self):
        # html table text to DataFrame
        html = self.html
        p = re.compile('<tr>(.*?)</tr>', re.DOTALL)
        trs = p.findall(html)
        
        p2 = re.compile('<td.*?>(.*?)</td>', re.DOTALL)
        tds = [p2.findall(tr) for tr in trs[1:]]
        df = pd.DataFrame({'证券代码': [td[0] for td in tds],
                           '简称': [td[1] for td in tds],
                           '公告标题': [td[2] for td in tds],
                           '公告时间': [td[3] for td in tds]})
        self.df_txt = df


    
    # 获得下载链接
    def get_link(self, txt):
        p_txt = '<a.*?attachpath="(.*?)".*?href="(.*?)".*?<span.*?>(.*?)</span>'
        p = re.compile(p_txt, re.DOTALL)
        matchObj = p.search(txt)
        attachpath = matchObj.group(1).strip()
        href       = matchObj.group(2).strip()
        title      = matchObj.group(3).strip()
        return([attachpath, href, title])

    def get_data(self):
        get_code = self.get_code
        get_time = self.get_time
        get_link = self.get_link
        # 
        df = self.df_txt
        codes = [get_code(td) for td in df['证券代码']]
        short_names = [get_code(td) for td in df['简称']]
        ahts = [get_link(td) for td in df['公告标题']]
        times = [get_time(td) for td in df['公告时间']]
        self.short_names = short_names
        #
        prefix = self.prefix
        prefix_href = self.prefix_href
        data = pd.DataFrame({'证券代码': codes,
                             '股票简称': short_names,
                             '公告标题': [aht[2] for aht in ahts],
                             'attachpath': [prefix + aht[0] for aht in ahts],
                             'href': [prefix_href + aht[1] for aht in ahts],
                             '公告时间': times})
        
        for index,row in data.iterrows():
            if re.search("摘要|取消|英文", row[2]) != None:
                data = data.drop(index=index)
        data = data.reset_index(drop = True)
        self.df_data = data
        return data


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 18 05:01:25 2023

@author: Napstablook
"""
from setuptools import find_packages, setup
# Package meta-data.
NAME = 'arfindata'
DESCRIPTION = "Python package for the aquisition and pre-treatment of China's listed companies' annual reports"
URL = 'https://github.com/napstablook04/ARfindata'
EMAIL = 'linkfr@163.com'    # 你的邮箱
AUTHOR = 'linkfr'     # 你的名字
REQUIRES_PYTHON = '>=3.6.0' # 项目支持的python版本
VERSION = '0.1.3'           # 项目版本号
REQUIRED = ['pandas','selenium','fitz','pdfplumber','requests']

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(),
    install_requires=REQUIRED,
    license="MIT"
)

